-----------------------------------------------------------------------------------------------
-- Client Lua Script for ForDummies
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ForDummies Module Definition
-----------------------------------------------------------------------------------------------
local ForDummies = {} 
local GeminiDB
local Utils
local Updates
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local n_VERSION = 1.1
local str_SUB_BUILD = "Build_"
local str_FORMAT_MACROTITLE = "ForDummies Build '%s'"
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ForDummies:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function ForDummies:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "ForDummies"
	local tDependencies = {
		"Macros",
		"Abilities",
		"ForDummies:Lib:Utils",
		"ForDummies:Lib:Updates",
		"Gemini:DB-1.0"
	}
	-- Register the addon
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ForDummies OnLoad
-----------------------------------------------------------------------------------------------
function ForDummies:OnLoad()
	-- Packages declaration
	GeminiDB = Apollo.GetPackage("Gemini:DB-1.0").tPackage
	Utils = Apollo.GetPackage("ForDummies:Lib:Utils").tPackage
	Updates = Apollo.GetPackage("ForDummies:Lib:Updates").tPackage
	-- Define the database
	local tDefaults = {} 
	tDefaults.char = {}
	tDefaults.char.tSwapper = {}
	tDefaults.char.tSwapper.bShow = false
	tDefaults.char.tSwapper.bMoveable = false
	tDefaults.char.tBuilds = {}
	tDefaults.char.tBuilds._index = 0
	tDefaults.char.tBuilds.tList = {}
	tDefaults.char.tBuilds.tList['*'] = {}
	tDefaults.char.tBuilds.tList['*'].nSelectedSlot = 1
	tDefaults.char.tBuilds.tList['*'].nStanceIndex = 1
	tDefaults.char.tBuilds.tList['*'].bListed = true
	self.db = GeminiDB:New(self, tDefaults)
	-- Register slash commands
	Apollo.RegisterSlashCommand("fd", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("fordummies", "OnSlashCommand", self)
	-- Register events
	Apollo.RegisterEventHandler("SpecChanged", "OnSpecChanged", self)
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	-- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ForDummies.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ForDummies OnDocLoaded
-----------------------------------------------------------------------------------------------
function ForDummies:OnDocLoaded()
	if self.xmlDoc == nil then 
		Apollo.AddAddonErrorText(self, "Could not load the xml document.")
		return 
	end
	-- Load the options window
	self:LoadOptions()
	-- Load the build swapper
	self.wndSwapper = Apollo.LoadForm(self.xmlDoc, "BuildSwapper", nil, self)
	-- Load dummy ability item slot 
	self.wndDummy = Apollo.LoadForm(self.xmlDoc, "DummyAbilityItem", nil, self)
	-- Check if version updates is needed
	self:TimersManager("UpdateToVersion", 2, false, "UpdateToVersion")
end

-----------------------------------------------------------------------------------------------
-- ForDummies Functions
-----------------------------------------------------------------------------------------------
-- On SlashCommand "/fd"
function ForDummies:OnSlashCommand( _, strArgs )
	local strSubBuild = "build="
	local tSwitches = {}
	tSwitches["updates"] = function() self:DisplayUpdates() end
	tSwitches["addbuild"] = function() self:NewBuild() end
	tSwitches[strSubBuild] = function() 
		local strId = string.sub(strArgs, string.len(strSubBuild) + 1)
		self:ActivateBuild(strId) 
	end
	-- Go through switches
	for strSwitch, fSwitch in pairs(tSwitches) do
		local bMatches = Utils:StartsWith(strArgs, strSwitch , false)
		-- If argument matches a switch
		if bMatches then 
			-- Call the right command
			fSwitch()
			return
		end
	end
	-- If argument doesn't match any switch, then default
	self:DisplayOptions()
end

-- On ForDummies menu
function ForDummies:OnConfigure()
	self:DisplayOptions()
end

-- On spec changed event
function ForDummies:OnSpecChanged()
	if self.strRequestBuildChange then
		self:TimersManager("ActivateBuildAfterSpecChange", 0.1, false, "ActivateBuildAfterSpecChange")
	end
end

-- Register to built-in window manager
function ForDummies:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndOptions, strName = "ForDummies_wndOptions"})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndSwapper, strName = "ForDummies_wndSwapper"})
	-- Manage the build swapper
	self:BuildSwapper()
end

-- Addon busy status manager
function ForDummies:SetBusy( b )
	-- Option window 	
	if self.wndOptions then	
		-- Tab menu
		self.wndOptions:FindChild("Frame:TabMenu"):Enable(not b) 
		-- Header
		self.wndOptions:FindChild("Frame:Header"):Enable(not b)
		-- Content
		self.wndOptions:FindChild("Frame:Bounds:Content"):Enable(not b)
	end
end

-- Load options window
function ForDummies:LoadOptions()
	-- Load the window form
	self.wndOptions = Apollo.LoadForm(self.xmlDoc, "Options", nil, self)
	-- Activate selected tab menu
	self.wndOptions:FindChild("Frame:TabMenu"):SetRadioSel("Options_TabMenu", 1)
end

-- Manage build swapper
function ForDummies:BuildSwapper()
	if not self.wndSwapper then return end
	local tSwapper = self.db.char.tSwapper
	-- Display or not the swapper 
	self.wndSwapper:Show(tSwapper.bShow, true)
	-- Set moveable or not the swapper
	self.wndSwapper:SetStyle("Moveable", tSwapper.bMoveable)
	-- Display or not the move helper
	self.wndSwapper:FindChild("MoveHelper"):Show(tSwapper.bMoveable, true)
	-- Check if current build is the last that was activated
	local strLastActivated = self.db.char.tBuilds.strLastActivated
	--if self:IsCurrentBuild(strLastActivated) then
	
	--end
end

-- Set swapper selected build icon
function ForDummies:SwapperSelectedBuildIcon( strId )
	if not self.wndSwapper then	return end
	local icon = self.wndSwapper:FindChild("Icon")
	self:SetStanceOrAbilityIcon(strId, icon)
end

-- Toggle swapper popup
function ForDummies:SwapperPopup( b )
	if not self.wndSwapper then return end
	self.wndSwapper:FindChild("PopoutFrame"):Show(b, true)
	local buildList = self.wndSwapper:FindChild("PopoutFrame:Bounds:BuildList")
	local tBuilds = self.db.char.tBuilds.tList
	-- Delete current build list
	buildList:DestroyChildren()
	-- Create build list
	local fOrder = function(t, a, b) 
		local nA = tonumber(string.sub(a, string.len(str_SUB_BUILD) + 1))
		local nB = tonumber(string.sub(b, string.len(str_SUB_BUILD) + 1))
		return nA < nB 
	end
	for strId, tBuild in Utils:spairs(tBuilds, fOrder) do
		if tBuild.bListed then
			-- Load build container
			local build = Apollo.LoadForm(self.xmlDoc, "SwapperBuildContainer", buildList, self)
			-- Set build id
			build:SetName(strId)
			-- Set build name
			build:FindChild("Name:strName"):SetText(tBuild.strName)
			-- Set related spec
			build:FindChild("ActionSet:nActionSet"):SetText(tBuild.nSpec)
			-- Set stance icon
			local iconSlot = build:FindChild("BuildIcon:SlotDisplay")
			self:SetStanceOrAbilityIcon(strId, iconSlot)
			-- Save build id
			build:FindChild("MouseCatcher"):SetData(strId)
		end
	end
	-- Sort the builds
	buildList:ArrangeChildrenVert(0)
end

-- When swapper popout frame is closed
function ForDummies:SwapperPopoutFrameClosed()
	if not self.wndSwapper then return end
	self.wndSwapper:FindChild("PopoutBtn"):SetCheck(false)
end

--
function ForDummies:SelectSwapperBuild( strId )
	-- Close the popout
	if self.wndSwapper then
		self.wndSwapper:FindChild("PopoutFrame"):Close()
	end
	-- Activate the build
	self:ActivateBuild(strId)
end

-- Display options window
function ForDummies:DisplayOptions()
	if not self.wndOptions then return end
	self.wndOptions:Show(true, true)
	self:OptionsTabManager()
end

-- Close options window
function ForDummies:CloseOptions()
	if not self.wndOptions then return end
	self.wndOptions:Show(false, true)
end

-- Manage Options Tab
function ForDummies:OptionsTabManager()
	if not self.wndOptions then return end
	local content = self.wndOptions:FindChild("Frame:Bounds:Content")
	local footer = self.wndOptions:FindChild("Frame:Footer")
	-- Destroy current content
	content:DestroyChildren()
	footer:DestroyChildren()
	-- Get selected tab
	local strTab = self.wndOptions:FindChild("Frame:TabMenu"):GetRadioSelButton("Options_TabMenu"):GetName()
	-- Display the selected content
	self.wndOptionsContent = Apollo.LoadForm(self.xmlDoc, string.format("Options_%s", strTab), content, self)
	self.wndOptionsFooter = Apollo.LoadForm(self.xmlDoc, string.format("Options_%sFooter", strTab), footer, self)
	-- Call the selected content manager
	self[string.format("%sOptions", strTab)](self)
	-- Call the selected footer
	self[string.format("%sOptionsFooter", strTab)](self)
end

-- Manage the builder options
function ForDummies:BuilderOptions()
	if not self.wndOptionsContent or self.wndOptionsContent:GetName() ~= "Options_Builder" then return end
	-- Destroy current build list
	self.wndOptionsContent:DestroyChildren()
	-- Display list of builds
	local tBuilds = self.db.char.tBuilds.tList
	local fOrder = function(t, a, b) 
		local nA = tonumber(string.sub(a, string.len(str_SUB_BUILD) + 1))
		local nB = tonumber(string.sub(b, string.len(str_SUB_BUILD) + 1))
		return nA < nB 
	end
	for strId, tBuild in Utils:spairs(tBuilds, fOrder) do
		-- Load build container
		local build = Apollo.LoadForm(self.xmlDoc, "BuildContainer", self.wndOptionsContent, self)
		-- Set build id
		build:SetName(strId)
		-- Set up build container
		self:SetBuildContainer(strId)
	end
	-- Sort the builds
	self.wndOptionsContent:ArrangeChildrenVert(0)
end

-- Manage the builder options footer
function ForDummies:BuilderOptionsFooter()
	local tSwapper = self.db.char.tSwapper
	-- Set visible check button 
	local buttonVisible = self.wndOptionsFooter:FindChild("Visible")
	buttonVisible:SetCheck(tSwapper.bShow)
	buttonVisible:SetText(tSwapper.bShow and "Visible" or "Hidden")
	-- Set moveable check button
	local buttonMoveable = self.wndOptionsFooter:FindChild("Moveable")
	buttonMoveable:SetCheck(tSwapper.bMoveable)
	buttonMoveable:SetText(tSwapper.bMoveable and "Moveable" or "Fixed")
end

-- Toggle build visible button
function ForDummies:ToggleBuildVisible( b )
	self.db.char.tSwapper.bShow = b
	self:BuilderOptionsFooter()
	self:BuildSwapper()
end

-- Toggle build moveable button
function ForDummies:ToggleBuildMoveable( b )
	self.db.char.tSwapper.bMoveable = b
	self:BuilderOptionsFooter()
	self:BuildSwapper()
end

-- Set up build container
function ForDummies:SetBuildContainer( strId )
	if not self.wndOptionsContent or self.wndOptionsContent:GetName() ~= "Options_Builder" then return end
	local build = self.wndOptionsContent:FindChild(strId)
	local tBuild = self:GetBuild(strId)
	local strSprite = "Anim_Inventory_New:sprInventory_NewItem"
	-- Set build name
	local name = build:FindChild("Bounds:Name:strName")
	name:SetText(tBuild.strName)
	name:SetData(strId)
	-- Set related spec
	build:FindChild("Bounds:ActionSet:nActionSet"):SetText(tBuild.nSpec)
	-- Set build stance slot
	local stanceSlot = build:FindChild("Bounds:SlotsContainer:SlotItem_0")
	local stanceMouseCatcher = stanceSlot:FindChild("SlotMouseCatcher")
	stanceSlot:FindChild("SlotDisplay"):SetSprite(self:GetStanceIcon(strId))
	stanceMouseCatcher:SetData(strId)
	if tBuild.nSelectedSlot == 0 then stanceMouseCatcher:SetSprite(strSprite)
	else stanceMouseCatcher:SetSprite() end
	-- Set build abilities slots
	for nSlot, nAbilityId in ipairs(tBuild.tActionSet)do
		local slot = build:FindChild(string.format("Bounds:SlotsContainer:SlotItem_%s", nSlot))
		local slotMouseCatcher = slot:FindChild("SlotMouseCatcher")
		-- Check if the slot is unlocked
		local bSlotBlocked = ActionSetLib.IsSlotUnlocked(nSlot - 1) ~= ActionSetLib.CodeEnumLimitedActionSetResult.Ok
		if bSlotBlocked then slot:FindChild("SlotLockedBlocker"):Show(true, true)
		else
			-- Set ability nId
			slot:FindChild("SlotDisplay"):SetAbilityId(nAbilityId)
			slotMouseCatcher:SetData(strId)
			-- Set ability tier token
			if tBuild.tTiers[nAbilityId] - 1 > 0 then
				local tierToken = slot:FindChild("SlotDisplay:nTier")
				tierToken:SetText(tBuild.tTiers[nAbilityId] - 1)
				tierToken:Show(true, true)
			end
		end
		-- Set selected build icon
		if nSlot == tBuild.nSelectedSlot then slotMouseCatcher:SetSprite(strSprite)
		else slotMouseCatcher:SetSprite() end
	end
	-- Check listed button
	local listed = build:FindChild("Bounds:Listed")
	listed:SetCheck(tBuild.bListed)
	listed:SetData(strId)
	-- Save build id to relevant places for later use
	build:FindChild("Activate"):SetData(strId)
	build:FindChild("Bounds:Modify"):SetData(strId)
	build:FindChild("Bounds:Macro"):SetData(strId)
	build:FindChild("Bounds:Delete"):SetData(strId)
end

-- Create a new or modify an existing build
function ForDummies:NewBuild( strId )
	local tBuild = {}
	local tBuildToModify = self:GetBuild(strId)
	if strId and not tBuildToModify then return end	 
	-- Set addon to busy
	self:SetBusy(true)
	-- Close CRB interface window if exist (canceling any ongoing modification)
	Apollo.GetAddon("Abilities"):OnCloseFinal()
	-- Get current stance
	tBuild.nStanceIndex = GameLib.GetCurrentClassInnateAbilityIndex()
	-- Get current spec
	tBuild.nSpec = AbilityBook.GetCurrentSpec()
	-- Get the current action set build
	tBuild.tActionSet = ActionSetLib.GetCurrentActionSet()
	for _ = 1, 4 do table.remove(tBuild.tActionSet, 9) end
	-- Get tier per ability
	tBuild.tTiers = {}
	for _, nAbilityId in ipairs(tBuild.tActionSet) do
		if nAbilityId > 0 then
			self.wndDummy:SetAbilityId(nAbilityId)
			local nAbilityTierId = self.wndDummy:GetAbilityTierId()
			tBuild.tTiers[nAbilityId] = GameLib.GetSpell(nAbilityTierId):GetTier()
		else tBuild.tTiers[nAbilityId] = 0 end
	end
	-- Create new build
	if not strId then
		-- Increment builds index 
		local nBuildIndex = Utils:IncrementTable(self.db.char.tBuilds, "_index")
		-- Set temporary name
		tBuild.strName = string.format("Build_%s", nBuildIndex)	
		-- Set selected icon
		tBuild.nSelectedSlot = 1
		-- Set listed
		tBuild.bListed = true
		-- Save the build
		self.db.char.tBuilds.tList[tBuild.strName] = tBuild
		-- Update build options list
		self:OptionsTabManager()
		-- Scroll down to new build if possible
		if self.wndOptionsContent and self.wndOptionsContent:GetName() == "Options_Builder" then 
			self.wndOptionsContent:SetVScrollPos(self.wndOptionsContent:GetVScrollRange())
		end
	-- Or modify existing one
	else
		-- Get build name
		tBuild.strName = tBuildToModify.strName	
		-- Get selected icon
		tBuild.nSelectedSlot = tBuildToModify.nSelectedSlot
		-- Get build listed
		tBuild.bListed = tBuildToModify.bListed
		-- Save the build
		self.db.char.tBuilds.tList[strId] = tBuild
		-- Update build container
		self:SetBuildContainer(strId)
	end
	-- Set addon to not busy
	self:SetBusy(false)
end

-- Activate build
function ForDummies:ActivateBuild( strId )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	-- set addon to busy
	self:SetBusy(true)
	-- Close CRB interface window if exist (canceling any ongoing modification)
	Apollo.GetAddon("Abilities"):OnCloseFinal()
	-- Change spec
	self.nTempSpec = AbilityBook.GetCurrentSpec()
	self.strRequestBuildChange = strId
	-- Change spec if required
	if tBuild.nSpec ~= self.nTempSpec then 
		AbilityBook.SetCurrentSpec(tBuild.nSpec) 
	-- Or go directly to build switch
	else self:ActivateBuildAfterSpecChange() end
end

-- Follow up after spec change has been made
function ForDummies:ActivateBuildAfterSpecChange()
	local strId = self.strRequestBuildChange
	local tBuild = self:GetBuild(strId)	
	if not tBuild then return end
	local tAbilityFilters = { 
		Spell.CodeEnumSpellTag.Assault, 
		Spell.CodeEnumSpellTag.Support, 
		Spell.CodeEnumSpellTag.Utility 
	}
	-- Check if all abilities are active
	local bIsValide = true
	for _, nAbilityId in ipairs(tBuild.tActionSet) do
		-- Anonymous function to cycle through filters 
		-- To be able to get out of the cycling as soon as we found the ability
		(function( nAbilityId )
			for _, nFilter in ipairs(tAbilityFilters) do
				for _, tAbility in ipairs(AbilityBook.GetAbilitiesList(nFilter)) do
					if tAbility.nId == nAbilityId then
						if not tAbility.bIsActive then bIsValide = false end
						return
					end
				end
			end
		end)(nAbilityId)
		if not bIsValide then break end
	end
	-- If an ability is not valide anymore (probably because of amp change)
	if not bIsValide then
		-- Warn the user
		Print("This build is not valide (check if you have not changed your amps since its creation)")
		-- Change back the spec to its former state if needed
		if self.nTempSpec ~= AbilityBook.GetCurrentSpec() then AbilityBook.SetCurrentSpec(self.nTempSpec) end
		-- Clean up
		self.nTempSpec = nil
		self.strRequestBuildChange = nil
		-- Set addon to not busy
		self:SetBusy(false)
		return
	end
	-- Reset all abilities tiers
	for _, nFilter in ipairs(tAbilityFilters) do
		for _, tAbility in ipairs(AbilityBook.GetAbilitiesList(nFilter)) do
			if tAbility.nCurrentTier > 1 and tAbility.bIsActive then 
				AbilityBook.UpdateSpellTier(tAbility.nId, 1) 
			end
		end
	end	
	-- Set build abilitis tier
	for nAbilityId, nTier in pairs(tBuild.tTiers) do
		AbilityBook.UpdateSpellTier(nAbilityId, nTier)
	end
	-- Try to change action set
	local tRestrictionCode = {
		pvp = ActionSetLib.CodeEnumLimitedActionSetResult.RestrictedInPVP,
		combat = ActionSetLib.CodeEnumLimitedActionSetResult.InCombat,
		missingTag = ActionSetLib.CodeEnumLimitedActionSetResult.MissingTag,
		dead = ActionSetLib.CodeEnumLimitedActionSetResult.PlayerIsDead,
		ok = ActionSetLib.CodeEnumLimitedActionSetResult.Ok
	}
	local tActionSet = ActionSetLib.GetCurrentActionSet()
	for i, nAbilityId in ipairs(tBuild.tActionSet) do tActionSet[i] = nAbilityId end
	local tResultInfo = ActionSetLib.RequestActionSetChanges(tActionSet)
	-- Manage errors
	if not tResultInfo or tResultInfo.eResult ~= tRestrictionCode.ok then
		if tResultInfo.eResult == tRestrictionCode.pvp or tResultInfo.eResult == tRestrictionCode.combat then
			Print("Builds can't be changed while in combat")
		elseif tResultInfo.eResult == tRestrictionCode.dead then
			Print("Builds can't be changed while dead")
		else
			Print("This set could not be activated")
		end
		-- Change back the spec to its former state if needed
		if self.nTempSpec ~= AbilityBook.GetCurrentSpec() then AbilityBook.SetCurrentSpec(self.nTempSpec) end
	else
		-- Change the stance
		GameLib.SetCurrentClassInnateAbilityIndex(tBuild.nStanceIndex)
		-- Warn user that the build was successfully changed
		Print(string.format("The build '%s' has been activated", tBuild.strName))
	end
	-- Set build icon on swapper tool

	-- Clean up
	self.nTempSpec = nil
	self.strRequestBuildChange = nil
	-- Set addon to not busy
	self:SetBusy(false)
end

-- Edit build name
function ForDummies:EditBuildName( strId, strName )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	-- Save old name for macro update
	local strOldName = tBuild.strName
	-- Save new build name
	tBuild.strName = strName 
	-- Modify associated macro
	self:NewMacro(strId, strOldName)
end

-- Select build icon
function ForDummies:SelectBuildIcon( strId, nSlot )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	local bSlotBlocked = ActionSetLib.IsSlotUnlocked(nSlot - 1) ~= ActionSetLib.CodeEnumLimitedActionSetResult.Ok
	if bSlotBlocked and nSlot ~= 0 then return end
	-- Save new icon selection 
	tBuild.nSelectedSlot = nSlot
	-- Update the associated macro icon if any
	self:NewMacro(strId, tBuild.strName)
	-- Redraw options window
	self:SetBuildContainer(strId)
	-- Set swapper selected build icon
	
end

-- Check if build is listed
function ForDummies:BuildListed( strId, bListed )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	-- Save the build listed
	tBuild.bListed = bListed  
end

-- Display modify build confirmation popup
function ForDummies:ModifyBuildConfirmation( strId )
	if not self.wndOptionsContent or self.wndOptionsContent:GetName() ~= "Options_Builder" then return end
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	-- Set addon to busy
	self:SetBusy(true)
	-- Display modify popup
	local popup = self.wndOptions:FindChild("Frame:Bounds:Builds_ModifyConfirmation")
	popup:Show(true, true)
	-- Display build name on modify window
	popup:FindChild("Frame:Text:BuildName"):SetText(tBuild.strName)
	-- Save build strId
	popup:FindChild("Frame:Confirm"):SetData(strId)
end

-- Modify an existing build
function ForDummies:ModifyBuild( strId )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	-- Modify the build
	self:NewBuild(strId)
	-- Update associated macro if exist
	local strName = tBuild.strName
	self:NewMacro(strId, strName)
end

-- Display delete build confirmation popup
function ForDummies:DeleteBuildConfirmation( strId )
	if not self.wndOptionsContent or self.wndOptionsContent:GetName() ~= "Options_Builder" then return end
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	-- Set addon to busy
	self:SetBusy(true)
	-- Display delete popup
	local popup = self.wndOptions:FindChild("Frame:Bounds:Builds_DeleteConfirmation")
	popup:Show(true, true)
	-- Display build name on delete window
	popup:FindChild("Frame:Text:BuildName"):SetText(tBuild.strName)
	-- Save build strId
	popup:FindChild("Frame:Confirm"):SetData(strId)
end

-- Delete build
function ForDummies:DeleteBuild( strId )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	local CRBmacros = Apollo.GetAddon("Macros")
	-- Check if associated macro exist
	local strTitle = string.format(str_FORMAT_MACROTITLE, tBuild.strName)
	local nMacroId = nil
	for _, tMacro in ipairs(MacrosLib.GetMacrosList()) do
		if tMacro.strName == strTitle then nMacroId = tMacro.nId end
	end
	-- Delete associated macro
	if nMacroId then 
		MacrosLib.DeleteMacro(nMacroId) 
		MacrosLib.SaveMacros()
	end
	-- Refresh CRB macro window only if window already opened
	if CRBmacros.wndMacros:IsShown() then CRBmacros:OnMacrosOn() end
	-- Delete the build
	self.db.char.tBuilds.tList[strId] = nil
	-- Redraw options window
	self:OptionsTabManager()
	-- Set addon to not busy
	self:SetBusy(false)
end

-- Manage the rotation options
function ForDummies:RotationOptions()
	
end

-- Manage the rotation options footer
function ForDummies:RotationOptionsFooter()
	
end

-- Update to version
function ForDummies:UpdateToVersion()
	local nDbVersion = self.db.char.nVersion
	local bUpdated = false
	local bNeedReload = false
	-- Update to version 1.1
	if not nDbVersion or nDbVersion < 1.1 then
		-- Remove former version system
		self.db.global.nVersion = nil
		-- Modify builds
		local tBuilds = self.db.char.tBuilds.tList
		for strId, tBuild in pairs(tBuilds) do
			-- Update macros
			self:NewMacro(strId, tBuild.strName)
		end
		bUpdated = true
	end
	-- Set new current version
	self.db.char.nVersion = n_VERSION
	-- Display updates window if bUpdated 
	if bUpdated then self:DisplayUpdates(bNeedReload) end
end

-- Display Updates window
function ForDummies:DisplayUpdates( bNeedReload ) 
	if self.wndUpdates then return end
	self.wndUpdates = Apollo.LoadForm(self.xmlDoc, "Updates", nil, self)
	local content = self.wndUpdates:FindChild("Frame:Content")
	-- Display the footer if need to reload
	if bNeedReload then
		local footer = self.wndUpdates:FindChild("Frame:Footer")
		local nLeft, nTop, nRight = content:GetAnchorOffsets()
		local _, _, _, nFooterBottom = footer:GetAnchorOffsets()
		local nFooterHeight = footer:GetHeight()
		local nBottom = nFooterBottom - nFooterHeight 
		content:SetAnchorOffsets(nLeft, nTop, nRight, nBottom)
		footer:Show(true, true)
	end
	-- Display updates
	local tUpdates = Updates:GetList()
	for strVersion, tUpdate in Utils:spairs(tUpdates, "desc") do
		-- Version
		local strFormatedVersion = string.format("Version %s:", strVersion)
		local nParent = content:AddNode(0, strFormatedVersion)
		-- Entries
		for _, tEntry in ipairs(tUpdate) do
			local strEntry = string.format("- %s", tEntry.text)
			local nEntry = content:AddNode(nParent, strEntry)
			-- Sub entries
			if tEntry.subs then
				for _, srtSub in ipairs(tEntry.subs) do
					local strFormatedSub = string.format("- %s", srtSub) 
					content:AddNode(nEntry, strFormatedSub)
				end
			end
		end
	end
end

-- Close updates window
function ForDummies:CloseUpdates()
	if not self.wndUpdates then return end
	self.wndUpdates:Destroy()
	self.wndUpdates = nil
end

-- Create a new or modify existing build macro
function ForDummies:NewMacro( strId, strName )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return end
	local CRBmacros = Apollo.GetAddon("Macros")
	local strIcon
	-- Set macro title
	local strTitle = string.format(str_FORMAT_MACROTITLE, strName or tBuild.strName)
	-- Check if macro already exist
	local nMacroId
	for _, tMacro in ipairs(MacrosLib.GetMacrosList()) do
		if tMacro.strName == strTitle then nMacroId = tMacro.nId end
	end
	-- If we want to modify but macro doesn't exist (user doesn't want a macro)
	-- or if we want to create a new macro but it already exist (we don't want to recreate)
	if strName and not nMacroId or not strName and nMacroId then return end
	-- Set stance icon
	if tBuild.nSelectedSlot == 0 then
		local nStanceIndex = (tBuild.nStanceIndex * 2) - 1
		strIcon = GameLib.GetClassInnateAbilitySpells().tSpells[nStanceIndex]:GetIcon()
	-- Or ability icon
	else
		local nAbilityId = tBuild.tActionSet[tBuild.nSelectedSlot]
		self.wndDummy:SetAbilityId(nAbilityId)
		local nAbilityTierId = self.wndDummy:GetAbilityTierId()
		strIcon = GameLib.GetSpell(nAbilityTierId):GetIcon()
	end
	-- Set macro content
	local strContent = "/fd build=" .. strId
	-- Create macro if not exist
	if not nMacroId then 
		nMacroId = MacrosLib.CreateMacro()
		-- Give information to user that macro has been created
		Print(string.format("The macro for build '%s' has been created", tBuild.strName))
		-- Popup/refresh CRB macro window
		CRBmacros:OnMacrosOn()
		-- Scroll the macro window
		local CRBmacrosList = CRBmacros.wndMacros:FindChild("List")
		local fUpdateMacro = function() CRBmacrosList:SetVScrollPos(CRBmacrosList:GetVScrollRange()) end
		self:TimersManager("NewMacro", 0.1, false, fUpdateMacro)
	end
	-- Set final macro title if updating
	strTitle = string.format(str_FORMAT_MACROTITLE, tBuild.strName)
	-- Set macro datas and save
	MacrosLib.SetMacroData(nMacroId, false, strTitle, strIcon, strContent)
	MacrosLib.SaveMacros()
	-- Refresh CRB macro window only if window already opened
	if CRBmacros.wndMacros:IsShown() then CRBmacros:OnMacrosOn() end
end

---------------------------------------------------------------------------------------------------
-- Options Functions
---------------------------------------------------------------------------------------------------
-- On close options window button
function ForDummies:Button_CloseOptions()
	self:CloseOptions()		
end

-- On display updates window button
function ForDummies:Button_DisplayUpdates()
	self:DisplayUpdates()
end

-- On check tab menu button
function ForDummies:Button_OptionsTabMenu()
	self:OptionsTabManager()
end

-- On modify build confirmation button
function ForDummies:Button_ModifyBuildConfirmation( wndHandler, wndControl )
	-- Hide popup
	self.wndOptions:FindChild("Frame:Bounds:Builds_ModifyConfirmation"):Show(false, true)
	-- If cancel
	if wndControl:GetName() == "Cancel" then
		-- Set addon to not busy
		self:SetBusy(false)
		return
	end
	-- Confirm modification
	self:ModifyBuild(wndControl:GetData())
end

-- On delete build confirmation button
function ForDummies:Button_DeleteBuildConfirmation( wndHandler, wndControl )
	-- Hide popup
	self.wndOptions:FindChild("Frame:Bounds:Builds_DeleteConfirmation"):Show(false, true)
	-- If cancel
	if wndControl:GetName() == "Cancel" then
		-- Set addon to not busy
		self:SetBusy(false)
		return
	end
	-- Confirm deletion
	self:DeleteBuild(wndControl:GetData())
end

------------------------------------------------------------------------------------------------
-- Updates Functions
---------------------------------------------------------------------------------------------------
-- On close updates window
function ForDummies:Button_CloseUpdates()
	self:CloseUpdates()
end

function ForDummies:Button_ReloadUI()
	RequestReloadUI()
end

---------------------------------------------------------------------------------------------------
-- Options_Builder Functions
---------------------------------------------------------------------------------------------------
-- On Add build button
function ForDummies:Button_AddBuild()
	self:NewBuild()
end

---------------------------------------------------------------------------------------------------
-- Options_BuilderFooter Functions
---------------------------------------------------------------------------------------------------
-- On build visible button
function ForDummies:Button_BuildVisible( wndHandler, wndControl )
	self:ToggleBuildVisible(wndControl:IsChecked())
end

-- On build moveable button
function ForDummies:Button_BuildMoveable( wndHandler, wndControl )
	self:ToggleBuildMoveable(wndControl:IsChecked())
end

---------------------------------------------------------------------------------------------------
-- BuildSwapper Functions
---------------------------------------------------------------------------------------------------
-- On swapper toggle button
function ForDummies:Button_SwapperToggle( wndHandler, wndControl )
	self:SwapperPopup(wndControl:IsChecked())
end

function ForDummies:Closed_SwapperPopoutFrame( wndHandler, wndControl )
	self:SwapperPopoutFrameClosed()
end

---------------------------------------------------------------------------------------------------
-- SwapperBuildContainer Functions
---------------------------------------------------------------------------------------------------

function ForDummies:Button_SelectSwapperBuild( wndHandler, wndControl )
	self:SelectSwapperBuild(wndControl:GetData())
end

---------------------------------------------------------------------------------------------------
-- BuildContainer Functions
---------------------------------------------------------------------------------------------------
-- On modify build button
function ForDummies:Button_ModifyBuild( wndHandler, wndControl )
	self:ModifyBuildConfirmation(wndControl:GetData())
end

-- On delete build button
function ForDummies:Button_DeleteBuild( wndHandler, wndControl )
	self:DeleteBuildConfirmation(wndControl:GetData())
end

-- On create build macro button
function ForDummies:Button_CreateMacro( wndHandler, wndControl )
	self:NewMacro(wndControl:GetData())
end

-- On select icon button
function ForDummies:Button_SelectBuildIcon( wndHandler, wndControl )
	local strName = wndControl:GetParent():GetName()
	local strSubSlotItem = "SlotItem_"
	local nSlot = tonumber(string.sub(strName, string.len(strSubSlotItem) + 1))
	self:SelectBuildIcon(wndControl:GetData(), nSlot)
end

-- On activate build button
function ForDummies:Button_ActivateBuild( wndHandler, wndControl )
	self:ActivateBuild(wndControl:GetData())
end

-- On check listed build button
function ForDummies:Button_BuildListed( wndHandler, wndControl )
	self:BuildListed(wndControl:GetData(), wndControl:IsChecked())
end

-- On edit build name changed
function ForDummies:Edit_BuildName( wndHandler, wndControl, strText )
	self:EditBuildName(wndControl:GetData(), strText)
end

---------------------------------------------------------------------------------------------------
-- Random Functions
---------------------------------------------------------------------------------------------------
-- Manage all the timers
function ForDummies:TimersManager( strName, nTime, bRepeat, xCallback)
	local strFunction = string.format("TimerFunction_%s", strName)
	local strTimer = string.format("Timer_%s", strName)
	-- Create the function which will manage the timer
	self[strFunction] = function()
		if type(xCallback) == "string" then self[xCallback](self)
		else xCallback() end
		-- Clean up the timer
		self[strFunction] = nil
		self[strTimer] = nil
	end
	-- Create the timer
	self[strTimer] = ApolloTimer.Create(nTime, bRepeat, strFunction, self)
end

-- Get db build
function ForDummies:GetBuild( strId )
	return self.db.char.tBuilds.tList[strId]
end

-- Get stance icon
function ForDummies:GetStanceIcon( strId )
	local tBuild = self:GetBuild(strId)
	local nStanceIndex = (tBuild.nStanceIndex * 2) - 1
	return GameLib.GetClassInnateAbilitySpells().tSpells[nStanceIndex]:GetIcon()
end

-- Set stance or ability icon
function ForDummies:SetStanceOrAbilityIcon( strId, iconSlot )
	local tBuild = self:GetBuild(strId)
	-- Set stance icon
	if tBuild.nSelectedSlot == 0 then
		local stanceIcon = self:GetStanceIcon(strId())
		iconSlot:SetSprite(stanceIcon)
	-- Or ability icon
	else
		local nAbilityId = tBuild.tActionSet[tBuild.nSelectedSlot]
		iconSlot:SetAbilityId(nAbilityId)
	end
end

-- Check if a specific build is equal to the current build setup
function ForDummies:IsCurrentBuild( strId )
	local tBuild = self:GetBuild(strId)
	if not tBuild then return false end
	-- Check if spec is equal
	if tBuild.nSpec ~= AbilityBook.GetCurrentSpec() then return false end
	-- Check if stance is equal
	if tBuild.nStanceIndex ~= GameLib.GetCurrentClassInnateAbilityIndex() then return false end
	-- Check if current actionset is equal
	local tCurrentActionSet = ActionSetLib.GetCurrentActionSet()
	for i = 1, 8 do 
		-- Check abilities
		if tCurrentActionSet[i] ~= tBuild.tActionSet[i] then return false end
		local nAbilityId = tCurrentActionSet[i]
		if nAbilityId > 0 then
			self.wndDummy:SetAbilityId(nAbilityId)
			local nAbilityTierId = self.wndDummy:GetAbilityTierId()
			local nAbilityTier = GameLib.GetSpell(nAbilityTierId):GetTier()
			-- Check tiers
			if nAbilityTier ~= tBuild.tTiers[nAbilityId] then return false end
		end
	end
	-- If everything is equal
	return true
end

-----------------------------------------------------------------------------------------------
-- ForDummies Instance
-----------------------------------------------------------------------------------------------
local ForDummiesInst = ForDummies:new()
ForDummiesInst:Init()


